
function [actuals,scores] = group_for_cross_validate(results, varargin)
	% Calculate statistics of a single run from run_cross_validate.
	% 
	% Usage:
	%   stats = stats_for_cross_validate(results, varargin)
	% 
	% Options:
	%    'score_type',X: how to do the scoring? Possible values:
	%       'per_repetition' (default)
	%       'per_fold'
	%       'per_row'
	%       'per_column'
	%       'automatic': depending on cv_type
	
	opts = struct(varargin{:});
	if ~isfield(opts,'score_type'), opts.score_type = 'per_repetition'; end
	
	if isequal(opts.score_type, 'automatic')
		if isequal(results.cv_type,'rows')
			opts.score_type = 'per_row';
		elseif isequal(results.cv_type,'cols')
			opts.score_type = 'per_column';
		elseif isequal(results.cv_type,'pairs')
			opts.score_type = 'per_repetition';
		else
			error('Unknown cv_type: %s',opts.cv_type);
		end
	end
	
	% load dataset
	data = load_dataset(results.dataset_name);
	
	% Number of 'parts'
	if isequal(opts.score_type, 'per_repetition')
		num_parts = 1;
	elseif isequal(opts.score_type, 'per_fold')
		num_parts = results.num_folds;
	elseif isequal(opts.score_type, 'per_row')
		num_parts = size(data.y,1);
	elseif isequal(opts.score_type, 'per_column')
		num_parts = size(data.y,2);
	else
		error('Unknown score_type: %s',opts.score_type);
	end
	
	% Group together the 'parts'
	actuals = cell(results.num_repetitions, num_parts);
	scores  = cell(results.num_repetitions, num_parts);
	for rep = 1:results.num_repetitions
		for part = 1:num_parts
			% Predictions / actual values
			if isequal(opts.score_type, 'per_repetition')
				actuals{rep,part} = data.y;
				scores {rep,part} = results.predictions{rep};
			elseif isequal(opts.score_type, 'per_fold')
				which = results.which_fold{rep} == part;
				actuals{rep,part} = data.y(which);
				scores {rep,part} = results.predictions{rep}(which);
			elseif isequal(opts.score_type, 'per_row')
				actuals{rep,part} = data.y(part,:);
				scores {rep,part} = results.predictions{rep}(part,:);
			elseif isequal(opts.score_type, 'per_column')
				actuals{rep,part} = data.y(:,part);
				scores {rep,part} = results.predictions{rep}(:,part);
			else
				error('Unknown score_type: %s',opts.score_type);
			end
		end
	end
end
