
function stats = stats_for_predictions(actuals, scores)
	stats = struct();
	stats.num_repetitions = size(actuals,1);
	stats.num_parts = size(actuals,2);
	stats.size = zeros(size(actuals));
	stats.auc  = zeros(size(actuals));
	stats.aupr = zeros(size(actuals));
	for i = 1:numel(actuals)
		if true
			pos = scores{i}(actuals{i} == 1);
			neg = scores{i}(actuals{i} == 0);
			[lt_neg,eq_neg,gt_neg, lt_pos,gt_pos] = with_lteqgt(pos,neg);
			stats.auc(i)  = sum(gt_neg + 0.5*eq_neg) / (numel(pos) * numel(neg));
			%stats.aupr(i) = mean((lt_pos+1) ./ (lt_pos+1+lt_neg+0.5*eq_neg));
			% The above is not quite correct, you want
			%   mean( u / (v+(0:eq_neq)) )
			% not
			%   u / (v + mean(0:eq_neq))
			stats.aupr(i) = mean((lt_pos+1) .* mean_one_over(lt_pos+1+lt_neg,eq_neg));
		end
		if true
			% Sort: good targets first or last
			[unused,order1] = sort(scores{i}(:) - 1e-13*actuals{i}(:),'descend');
			[unused,order2] = sort(scores{i}(:) + 1e-13*actuals{i}(:),'descend');
			sorted_actuals1 = actuals{i}(order1);
			sorted_actuals2 = actuals{i}(order2);
			stats.auc_lo(i)  = calculate_auc(sorted_actuals1(:));
			stats.aupr_lo(i) = calculate_aupr(sorted_actuals1(:));
			stats.auc_hi(i)  = calculate_auc(sorted_actuals2(:));
			stats.aupr_hi(i) = calculate_aupr(sorted_actuals2(:));
		end
		if false
			% What we actually want is the expected auc/aupr over random permutations among items with the same value
			num_perms = 100;
			aucs_perm  = zeros(1,num_perms);
			auprs_perm = zeros(1,num_perms);
			for p = 1:num_perms
				perm = randperm(numel(actuals{i}));
				[unused,order] = sort(scores{i}(perm),'descend');
				sorted_actuals = actuals{i}(perm)(order);
				aucs_perm(p)  = calculate_auc(sorted_actuals(:));
				auprs_perm(p) = calculate_aupr(sorted_actuals(:));
			end
			stats.auc_perm(i)  = mean(aucs_perm); % Mean of auc_lo and auc_hi is expected value of this
			stats.aupr_perm(i) = mean(auprs_perm); % mean of aupr_lo and aupr_hi is NOT near expected value of this
			% Good estimators?
			stats.auc(i)  = (stats.auc_lo(i) + stats.auc_hi(i)) / 2;
			%stats.auc(i)  = stats.auc_perm(i);
			%stats.aupr(i) = (stats.aupr_lo(i) + stats.aupr_hi(i)) / 2;
			stats.aupr(i) = stats.aupr_perm(i);
		elseif false
			% Sort
			[unused,order] = sort(scores{i}(:),'descend');
			sorted_actuals = actuals{i}(order);
			% Calculate auc etc.
			stats.auc(i)  = calculate_auc(sorted_actuals(:));
			stats.aupr(i) = calculate_aupr(sorted_actuals(:));
		end
		if false
			[m_auc,s_auc]   = bootstrap(@calculate_auc, actuals{i}(:),scores{i}(:));
			[m_aupr,s_aupr] = bootstrap(@calculate_aupr,actuals{i}(:),scores{i}(:));
			stats.bootstrap_mean_auc(i)  = m_auc;
			stats.bootstrap_mean_aupr(i) = m_aupr;
			stats.bootstrap_std_auc(i)  = s_auc;
			stats.bootstrap_std_aupr(i) = s_aupr;
		end
		stats.size(i) = numel(actuals{i});
	end
	
	% Means, weighted by part size
	stats.total_size = sum(stats.size(1,:));
	stats.auc_mean1  = sum(stats.auc  .* stats.size,2) / stats.total_size;
	stats.aupr_mean1 = sum(stats.aupr .* stats.size,2) / stats.total_size;
	
	stats.auc_mean  = mean(stats.auc(:));
	stats.aupr_mean = mean(stats.aupr(:));
	stats.auc_stddev  = std(stats.auc(:));
	stats.aupr_stddev = std(stats.aupr(:));
end

function mov = mean_one_over(v,n)
	% calculate mean(1 ./ (v+0:n))
	
	% Correct closed formula is
	%   Sum[1/(v + i), {i, 0, n}] / (n+1)
	%   (- PolyGamma[0, v] + PolyGamma[0, n + v + 1]) / (n+1)
	% below is an approximation with polyamma(0,x) ~= log(x) for the n>=2 part,
	% which is what you get when replacing the sum with an integral.
	ov = 1 ./ v;
	max_k = 10;
	for k=1:max_k
		ov(n >= k) = ov(n >= k) + 1 ./ (v(n >= k) + k);
	end
	ov(n > max_k) = ov(n > max_k) + log(v(n > max_k) + n(n > max_k) + 1) - log(v(n > max_k) + max_k + 1);
	mov = ov ./ (n + 1);
end
