
function [out,eq,gt, ltA,gtA] = with_lteqgt(A,B,f)
	% Usage:
	%   out = with_lteqgt(A,B,f)
	%   [lt,eq,gt] = with_lteqgt(A,B)
	%
	% For each item x in A, calculate f(lt,eq,gt)
	% where
	%   lt is the number of items y in B such that are x < y
	%   eq is the number of items y in B such that are x == y
	%   gt is the number of items y in B such that are x > y
	
	% Fast version
	combined = [A(:);B(:);B(:)];
	labels   = [zeros(numel(A),1);-ones(numel(B),1);ones(numel(B),1)];
	[sr,order] = sortrows([combined,labels]);
	as = zeros(size(A));
	as(order(sr(:,2)==0)) = find(sr(:,2)==0); % elements that come from A, in the order in which they appear i A
	gts = cumsum(sr(:,2) == 1);  % the 1s get sorted after equal items from A
	ges = cumsum(sr(:,2) == -1); % the -1s get sorted before equal items from A
	gesA = cumsum(sr(:,2) == 0);
	
	if nargin >= 3
		out = f(numel(B)-ges(as), ges(as)-gts(as), gts(as));
	else
		out = numel(B)-ges(as);
		eq  = ges(as)-gts(as);
		gt  = gts(as);
		gtA = gesA(as)-1;
		ltA = numel(A)-gesA(as);
	end
	
	if false
		% Slow version
		lts_slow = zeros(size(A));
		eqs_slow = zeros(size(A));
		gts_slow = zeros(size(A));
		for i = 1:numel(A)
			for j = 1:numel(B)
				if A(i) < B(j)
					lts_slow(i) = lts_slow(i)+1;
				elseif A(i) > B(j)
					gts_slow(i) = gts_slow(i)+1;
				else
					eqs_slow(i) = eqs_slow(i)+1;
				end
			end
		end
		out_slow = f(lts_slow, eqs_slow, gts_slow);
		assert(isequal(out,out_slow));
	end
end
